#ifndef GAME_H
#define GAME_H

#include "client/input/Control.h"
#include "client/math/Camera.h"
#include "client/utils/Clock.h"
#include "client/rendering/RenderSettings.h"
#include "client/rendering/Renderer.h"
#include "client/rendering/Mesh.h"
#include "client/rendering/FileTexture.h"
#include "client/rendering/FontRenderer.h"
#include "common/world/World.h"
#include "common/block/BlockRegistry.h"
#include "rendering/renderer/WorldRenderer.h"

class Game final {
public:
    Game(const Control& control, const Camera& camera, Ray& ray, const Clock& fps, const Clock& tps, 
            RenderSettings& renderSettings);

    void tick();
    void renderWorld(float lag, Renderer& renderer) const;
    void renderTextOverlay(float lag, Renderer& renderer, FontRenderer& fr) const;
    
    bool isRunning() const;

private:
    const Control& control;
    const Camera& camera;
    Ray& ray;
    const Clock& fps;
    const Clock& tps;
    RenderSettings& renderSettings;

    float lengthAngle;
    float widthAngle;
    Vector pos;
    
    BlockRegistry blockRegistry;
    World world;
    
    WorldRenderer worldRenderer;
};

#endif